# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Form Objects for Customer Orders
"""

from __future__ import unicode_literals

from rattail import enum
from rattail.db import model

import formencode
from formencode import validators

from tailbone.forms.validators import ValidCustomer, ValidProduct, ValidUser


class ValidCustomerInfo(validators.FormValidator):
    """
    Custom validator to ensure we have either a proper customer reference, or
    at least a customer name, when creating a new customer order.
    """

    def validate_python(self, field_dict, state):
        if not field_dict['customer'] and not field_dict['customer_name']:
            raise formencode.Invalid("Customer name is required", field_dict, state)


class NewCustomerOrderItem(formencode.Schema):
    """
    Form schema to which individual items on a new customer order must adhere.
    """
    allow_extra_fields = True

    product = ValidProduct()
    product_description = validators.NotEmpty()

    quantity = validators.Int()
    unit_of_measure = validators.OneOf(enum.UNIT_OF_MEASURE)
    discount = validators.Int()

    notes = validators.String()


class NewCustomerOrder(formencode.Schema):
    """
    Form schema for creating a new customer order.
    """
    allow_extra_fields = True
    pre_validators = [formencode.NestedVariables()]

    user = formencode.Pipe(validators=[
        validators.NotEmpty(),
        ValidUser()])

    customer = ValidCustomer()
    customer_name = validators.String()
    customer_phone = validators.NotEmpty()

    products = formencode.ForEach(NewCustomerOrderItem())

    chained_validators = [ValidCustomerInfo()]
