# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views for creating new Customer Orders
"""

from __future__ import unicode_literals

from rattail import enum
from rattail.db import model
from rattail.db import custorders

from pyramid.renderers import render_to_response
from pyramid_simpleform import Form

from tailbone.db import Session
from tailbone.forms import renderers, FormRenderer
from tailbone.forms.custorders import NewCustomerOrder
from tailbone.views import View


class NewCustomerOrder(View):
    """
    View for creating a new customer order.
    """

    def __init__(self, request):
        super(NewCustomerOrder, self).__init__(request)
        self.handler = custorders.get_current_handler(Session())

    def __call__(self):
        """
        Use custom view instead of normal CRUD when creating new customer orders.
        """
        form = Form(self.request, schema=NewCustomerOrder)
        if self.request.method == 'POST':
            if form.validate():

                user = form.data['user']

                order = model.CustomerOrder()
                order.customer = form.data['customer']
                if order.customer:
                    order.customer_name = order.customer.name
                else:
                    order.customer_name = form.data['customer_name']
                order.customer_phone = form.data['customer_phone']

                # user = form.data['user']
                # customer = form.data['customer']
                # if customer:
                #     customer_name = customer.name
                # else:
                #     customer_name = form.data['customer_name']
                # customer_phone = form.data['customer_phone']

                # if form.data.get('update_customer_phone') == 'true':
                #     data = {'user': user, 'customer': customer, 'phone': customer_phone}
                #     send_email(self.rattail_config, 'specialorders_customer_phone_update', data)

                # order_uuids = []
                for data in form.data['products']:
                    # order = model.CustomerOrder()
                    # order.customer = customer
                    # order.customer_name = customer_name
                    # order.customer_phone = customer_phone

                    item = model.CustomerOrderItem()

                    # Set these first so extended price may be calculated below.
                    item.quantity = data['quantity']
                    item.unit_of_measure = data['unit_of_measure']
                    item.discount = data['discount']

                    item.product = data['product']
                    if item.product:
                        if item.product.brand:
                            item.product_brand = item.product.brand.name
                        item.product_description = item.product.description
                        item.product_size = item.product.size
                        item.product_unit_of_measure = item.product.unit_of_measure
                        item.product_case_pack = item.product.case_pack
                        if item.product.cost:
                            item.product_unit_cost = item.product.cost.unit_cost
                        # item.unit_price = order.product.regular_price.price / order.product.regular_price.multiple
                        # item.extended_price = order.calculate_extended_price()
                    else:
                        item.product_description = data['product_description']

                    item.status = self.handler.default_created_order_item_status
                    order.items.append(item)

                Session.add(order)
                return render_to_response('json', {'result': 'success'}, request=self.request)

            else:
                return render_to_response('json', {'result': form.errors}, request=self.request)

        # ambiguous_product_fields = parse_list(self.rattail_config.require(
        #         u'dtail', u'specialorders.ambiguous_product_fields'))

        return {
            'form': FormRenderer(form),
            # 'default_discount': get_default_discount(self.rattail_config),
            # 'ambiguous_product_fields': ambiguous_product_fields,
            }


def uom_for_product(product):
    """
    Determine the default "unit" UOM for the given product.  Will always be
    either "Pound" or "Each".
    """
    if product.weighed:
        return enum.UNIT_OF_MEASURE_POUND
    return enum.UNIT_OF_MEASURE_EACH


def uom_choices_for_product(product=None):
    """
    Generate a UOM enumeration customized for the given product.
    """

    if product:
        uom = uom_for_product(product)
        uoms = [(uom, enum.UNIT_OF_MEASURE[uom])]
        if product.case_pack and product.case_pack > 1:
            uoms.append((
                    enum.UNIT_OF_MEASURE_CASE,
                    "{0} ({1} {2})".format(
                        enum.UNIT_OF_MEASURE[enum.UNIT_OF_MEASURE_CASE],
                        product.case_pack,
                        enum.UNIT_OF_MEASURE[uom])))

    else:
        uoms = [
            (enum.UNIT_OF_MEASURE_EACH, enum.UNIT_OF_MEASURE[enum.UNIT_OF_MEASURE_EACH]),
            (enum.UNIT_OF_MEASURE_POUND, enum.UNIT_OF_MEASURE[enum.UNIT_OF_MEASURE_POUND]),
            (enum.UNIT_OF_MEASURE_CASE, enum.UNIT_OF_MEASURE[enum.UNIT_OF_MEASURE_CASE]),
            ]

    return uoms


def get_product_info(config, uuid):
    """
    Generic, sort of, function to retrieve a dictionary of info for the product
    specified by ``uuid``.
    """
    product = Session.query(model.Product).get(uuid) if uuid else None
    if not product:
        return {}

    info = {
        'product_upc': unicode(product.upc),
        'product_brand': product.brand.name if product.brand else '',
        'product_description': product.description,
        'product_size': product.size,
        'product_case_pack': product.case_pack or 1,
        # 'units_of_measure': uom_choices_for_product(product),
        # u'default_discount': get_default_discount(config, product),
        }

    prices = {}
    if product.regular_price:
        price = product.regular_price.price
        prices[uom_for_product(product)] = "{0:0.2f}".format(price)
        if info['product_case_pack'] > 1:
            prices[enum.UNIT_OF_MEASURE_CASE] = "{0:0.2f}".format(
                price * info['product_case_pack'])
    info['prices'] = prices
    return info


def product_info(request):
    return get_product_info(request, request.GET.get('uuid'))


def includeme(config):

    config.add_route('custorders.new', '/custorders/new')
    config.add_view(NewCustomerOrder, route_name='custorders.new',
                    renderer='/custorders/new.mako',
                    permission='custorders.create')

    config.add_route('custorders.product_info', '/custorders/product-info')
    config.add_view(product_info, route_name='custorders.product_info',
                    renderer='json')
